// packages/babel/src/transforms/callExpression.ts
import { types as t2 } from "@babel/core";

// packages/babel/src/utils.ts
import { template, types as t } from "@babel/core";
import { addNamed } from "@babel/helper-module-imports";
var buildImport = template("import { %%specifier%% as %%alias%% } from %%source%%;");
var buildModuleExport = template("module.exports = %%nodes%%");
var buildNamedExport = template("export { %%ident%% }");
var buildComponentStyle = template(`
const %%className%% = %%recipeImport%%(%%styles%%);
`);
var buildComponent = template(`const %%component%% = /* @__PURE__ */ %%styledImport%%(%%tag%%, %%className%%);`);
function invariant(cond, message) {
  if (!cond) {
    throw new Error(message);
  }
}
function registerImportMethod(path, name, moduleName = "@macaron-css/core") {
  const imports = path.scope.getProgramParent().macaronData.imports || (path.scope.getProgramParent().macaronData.imports = /* @__PURE__ */ new Map());
  if (!imports.has(`${moduleName}:${name}`)) {
    let id = addNamed(path, name, moduleName);
    imports.set(`${moduleName}:${name}`, id);
    return id;
  } else {
    let iden = imports.get(`${moduleName}:${name}`);
    return t.cloneNode(iden);
  }
}
function getNearestIdentifier(path) {
  let currentPath = path;
  while (currentPath.parentPath !== null) {
    if (currentPath.isIdentifier()) {
      return currentPath;
    }
    let id = currentPath.get("id");
    if (!Array.isArray(id)) {
      if (id.isIdentifier())
        return id;
      if (id.isArrayPattern()) {
        for (const el of id.get("elements")) {
          if (el.isIdentifier())
            return el;
        }
      }
    }
    let key = currentPath.get("key");
    if (!Array.isArray(key) && key.isIdentifier()) {
      return key;
    }
    currentPath = currentPath.parentPath;
  }
  return null;
}
var extractionAPIs = [
  "style",
  "styleVariants",
  "globalStyle",
  "createTheme",
  "createGlobalTheme",
  "createThemeContract",
  "createGlobalThemeContract",
  "assignVars",
  "createVar",
  "fallbackVar",
  "fontFace",
  "globalFontFace",
  "keyframes",
  "globalKeyframes",
  "style",
  "styleVariants",
  "globalStyle",
  "createTheme",
  "createGlobalTheme",
  "createThemeContract",
  "createGlobalThemeContract",
  "assignVars",
  "createVar",
  "fallbackVar",
  "fontFace",
  "globalFontFace",
  "keyframes",
  "globalKeyframes",
  "recipe"
];

// packages/babel/src/transforms/callExpression.ts
function transformCallExpression(callPath, _callState) {
  const callee = callPath.get("callee");
  if (!extractionAPIs.some((api) => callee.referencesImport("@macaron-css/core", api))) {
    return;
  }
  const programParent = callPath.scope.getProgramParent();
  const nearestIdentifier = getNearestIdentifier(callPath);
  const ident = nearestIdentifier ? programParent.generateUidIdentifier(`$$macaron_${nearestIdentifier.node.name}`) : programParent.generateUidIdentifier("$$macaron_identifier");
  const importedIdent = registerImportMethod(callPath, ident.name, programParent.macaronData.cssFile);
  const declaration = t2.exportNamedDeclaration(t2.variableDeclaration("var", [
    t2.variableDeclarator(ident, t2.cloneNode(callPath.node))
  ]));
  callPath.traverse({
    Expression(expressionPath, state) {
      if (!expressionPath.isIdentifier())
        return;
      const binding = callPath.scope.getBinding(expressionPath);
      if (!binding || programParent.macaronData.bindings.includes(binding.path))
        return;
      programParent.macaronData.bindings.push(binding.path);
      programParent.macaronData.nodes.push({
        type: "binding",
        node: t2.cloneNode(findRootBinding(binding.path))
      });
    }
  });
  programParent.macaronData.nodes.push({
    type: "style",
    export: declaration,
    shouldReExport: false
  });
  callPath.replaceWith(importedIdent);
}
function findRootBinding(path) {
  var _a;
  let node;
  if (!("parent" in path) || ((_a = path.parentPath) == null ? void 0 : _a.isProgram())) {
    node = path.node;
  } else {
    node = path.parent;
  }
  return t2.cloneNode(node);
}

// packages/babel/src/transforms/postprocess.ts
import { types as t3 } from "@babel/core";
import generate from "@babel/generator";
function postprocess(path, state) {
  const programParent = path.scope;
  const program = t3.program(programParent.macaronData.nodes.map((node) => node.type === "style" ? node.export : node.node));
  const cssExtract = generate(program).code;
  state.opts.result = [programParent.macaronData.cssFile, cssExtract];
}

// packages/babel/src/visitors/styled-components.ts
import { types as t4 } from "@babel/core";
function styledComponentsVisitor(programPath) {
  return {
    VariableDeclaration(variablePath) {
      const declarations = variablePath.get("declarations");
      if (declarations.length < 1)
        return;
      const decl = declarations[0].get("init");
      const id = declarations[0].get("id");
      if (!decl.isCallExpression())
        return;
      if (!id.isIdentifier())
        return;
      const isSolidAdapter = decl.get("callee").referencesImport("@macaron-css/solid", "styled");
      const isReactAdapter = decl.get("callee").referencesImport("@macaron-css/react", "styled");
      const isImportedFromAdapter = isSolidAdapter || isReactAdapter;
      const runtimeImport = isSolidAdapter ? "@macaron-css/solid/runtime" : "@macaron-css/react/runtime";
      if (!isImportedFromAdapter) {
        return;
      }
      invariant(decl.get("arguments").length === 2, "Wrong arguments to `styled`");
      const [tag, styles] = decl.get("arguments");
      const className = variablePath.scope.generateUidIdentifier(id.node.name.toLowerCase());
      const styledIdent = programPath.scope.generateUidIdentifier("$$styled");
      const recipeIdent = programPath.scope.generateUidIdentifier("recipe");
      const stmts = [
        buildImport({
          specifier: t4.identifier("$$styled"),
          alias: styledIdent,
          source: t4.stringLiteral(runtimeImport)
        }),
        buildImport({
          specifier: t4.identifier("recipe"),
          alias: recipeIdent,
          source: t4.stringLiteral("@macaron-css/core")
        }),
        buildComponentStyle({
          recipeImport: recipeIdent,
          styles: styles.node,
          className
        })
      ];
      programPath.unshiftContainer("body", stmts);
      const callExpression = t4.callExpression(styledIdent, [
        t4.cloneNode(tag.node),
        className
      ]);
      t4.addComment(callExpression, "leading", " @__PURE__ ");
      const component = t4.variableDeclaration("const", [
        t4.variableDeclarator(t4.identifier(id.node.name), callExpression)
      ]);
      variablePath.replaceWith(component);
      variablePath.scope.crawl();
    }
  };
}

// packages/babel/src/transforms/preprocess.ts
import hash from "@emotion/hash";
function preprocess(path, state) {
  state.dependentNodes = /* @__PURE__ */ new Set();
  state.styledNodes = [];
  path.scope.macaronData = {
    imports: /* @__PURE__ */ new Map(),
    styles: [],
    cssFile: `extracted_${hash(path.toString())}.css.ts`,
    nodes: [],
    bindings: []
  };
  path.traverse(styledComponentsVisitor(path), state);
}

// packages/babel/src/index.ts
function macaronBabelPlugin() {
  return {
    name: "macaron-css-babel",
    visitor: {
      Program: {
        enter: preprocess,
        exit: postprocess
      },
      CallExpression: transformCallExpression
    }
  };
}
export {
  macaronBabelPlugin
};
//# sourceMappingURL=index.mjs.map